﻿using Microsoft.Extensions.Configuration;
using System.IO;
using System.Net.Sockets;
using System.Net;
using System.Text;
using System.Diagnostics;

namespace CS2ServerOnlineStatus
{
    public class ServerStatusService
    {
        public readonly ServerSettings Settings;

        public ServerStatusService()
        {
            var builder = new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile("appsettings.json", optional: false, reloadOnChange: true);
            IConfigurationRoot configuration = builder.Build();
            Settings = configuration.GetSection("ServerSettings").Get<ServerSettings>()
                ?? throw new InvalidOperationException("Server settings not found in appsettings.json.");
        }

        public async Task<ServerData> GetServerDataAsync()
        {
            string resolvedAddress = await GetResolvedServerAddressAsync();
            using UdpClient client = new();
            try
            {
                client.Connect(resolvedAddress, Settings.Port);
            }
            catch (Exception ex)
            {
                throw new Exception($"Failed to connect to {resolvedAddress}:{Settings.Port}: {ex.Message}", ex);
            }

            byte[] queryPacket = new byte[]
            {
                0xFF, 0xFF, 0xFF, 0xFF, 0x54, 0x53, 0x6F, 0x75, 0x72, 0x63, 0x65,
                0x20, 0x45, 0x6E, 0x67, 0x69, 0x6E, 0x65, 0x20, 0x51, 0x75, 0x65, 0x72, 0x79, 0x00
            };

            await client.SendAsync(queryPacket, queryPacket.Length);

            UdpReceiveResult receiveResult;
            try
            {
                var receiveTask = client.ReceiveAsync();
                if (await Task.WhenAny(receiveTask, Task.Delay(5000)) == receiveTask)
                {
                    receiveResult = receiveTask.Result;
                }
                else
                {
                    throw new Exception("Server response timed out after 5 seconds.");
                }
            }
            catch (Exception ex)
            {
                throw new Exception($"Failed to receive data: {ex.Message}", ex);
            }

            byte[] response = receiveResult.Buffer;
            if (response.Length <= 6)
            {
                throw new Exception("Server response too short.");
            }

            int offset = 6;
            string[] parts = new string[5];
            int partIndex = 0;
            int start = offset;
            for (int i = offset; i < response.Length && partIndex < 4; i++)
            {
                if (response[i] == 0)
                {
                    parts[partIndex] = Encoding.ASCII.GetString(response, start, i - start);
                    partIndex++;
                    start = i + 1;
                }
            }
            parts[4] = Encoding.ASCII.GetString(response, start, response.Length - start);

            int packetStart = start;
            byte[] packet = new byte[response.Length - packetStart];
            Array.Copy(response, packetStart, packet, 0, packet.Length);

            return new ServerData
            {
                ServerName = parts[0],
                Map = parts[1],
                Game = parts[2],
                Description = parts[3],
                AppID = BitConverter.ToUInt16(packet, 0).ToString(),
                Players = packet.Length > 2 ? packet[2].ToString() : "0",
                PlayersMax = packet.Length > 3 ? packet[3].ToString() : "0",
                Bots = packet.Length > 4 ? packet[4].ToString() : "0",
                Dedicated = packet.Length > 5 ? ((char)packet[5]).ToString() : string.Empty,
                OS = packet.Length > 6 ? ((char)packet[6]).ToString() : string.Empty,
                Password = packet.Length > 7 ? packet[7].ToString() : "0",
                VAC = packet.Length > 8 ? packet[8].ToString() : "0"
            };
        }

        public async Task<string> GetResolvedServerAddressAsync()
        {
            try
            {
                IPAddress[] addresses = await Dns.GetHostAddressesAsync(Settings.Address);
                return addresses.Length > 0 ? addresses[0].ToString() : Settings.Address;
            }
            catch (Exception ex)
            {
                throw new Exception($"Failed to resolve address {Settings.Address}: {ex.Message}", ex);
            }
        }

        public void LaunchBrowser(string url)
        {
            if (!File.Exists(Settings.BrowserExecutablePath))
            {
                throw new FileNotFoundException($"Browser executable not found at: {Settings.BrowserExecutablePath}");
            }

            // Ignore OpenIncognito for Discord URL
            bool useIncognito = Settings.OpenIncognito && url != "discord.com/app";

            string arguments = Settings.BrowserName.ToLower() switch
            {
                "chrome" => $"--new-window {(useIncognito ? "-incognito " : "")}{url}",
                "opera" => $"{(useIncognito ? "--private " : "")}{url}",
                "edge" => $"{(useIncognito ? "--inprivate " : "")}{url}",
                "firefox" => $"{(useIncognito ? "-private-window " : "")}{url}",
                _ => throw new ArgumentException($"Unsupported browser: {Settings.BrowserName}")
            };

            Process.Start(
                new ProcessStartInfo
                {
                    FileName = Settings.BrowserExecutablePath,
                    Arguments = arguments,
                    UseShellExecute = true
                }
            );
        }
    }
}
